"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useSourceLegacy = void 0;
const react_1 = require("react");
const use_subscription_1 = require("use-subscription");
const source_1 = require("../source");
const MissingToken = Symbol();
const useSourceLegacy = (maybeSource, options = {}) => {
    const hasDefaultValue = 'defaultValue' in options;
    const { defaultValue, startTransition } = options;
    const [core, select] = maybeSource || source_1.nullSource;
    const getCurrentValue = (0, react_1.useCallback)(() => {
        const vector = core[0]();
        if (!hasDefaultValue && !vector.length) {
            throw (0, source_1.getSnapshotPromise)([core, source_1.identitySelector]);
        }
        return vector.length ? select(vector[0]) : MissingToken;
    }, [core, hasDefaultValue, select]);
    const subscribe = (0, react_1.useMemo)(() => startTransition
        ? (callback) => core[1](() => {
            startTransition(callback);
        })
        : core[1], [core, startTransition]);
    const subscription = (0, react_1.useMemo)(() => ({
        getCurrentValue,
        subscribe,
    }), [getCurrentValue, subscribe]);
    // useSubscription doesn't suspend if its value changes to a missing value.
    // This hack normalizes it to act the same as useMutableSource.
    const [, setState] = (0, react_1.useState)();
    (0, react_1.useEffect)(() => {
        if (!hasDefaultValue) {
            return subscribe(() => {
                if (!(0, source_1.hasSnapshot)([core])) {
                    setState((() => {
                        throw (0, source_1.getSnapshotPromise)([core, source_1.identitySelector]);
                    }));
                }
            });
        }
    }, [hasDefaultValue, core, subscribe]);
    const value = (0, use_subscription_1.useSubscription)(subscription);
    return value === MissingToken || maybeSource === null ? defaultValue : value;
};
exports.useSourceLegacy = useSourceLegacy;
