"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createState = void 0;
const vectorState_1 = require("./vectorState");
const defaultIsEqual = (x, y) => x === y;
function createState(
// Doesn't accept a setter function, as it's not a hook that stores state
// between renders and thus it wouldn't make sense to do so.
initialState, 
// If provided, the state will only be updated if this function indicates
// that it is not equal to the current state.
isEqual = defaultIsEqual) {
    const areVectorsEqual = (x, y) => x.length === y.length && isEqual(x[0], y[0]);
    const [source, setVectorState, sealState] = (0, vectorState_1.createVectorState)(arguments.length === 0 ? [] : [initialState], areVectorsEqual);
    const setState = (stateOrUpdater) => {
        if (typeof stateOrUpdater === 'function') {
            const updater = stateOrUpdater;
            setVectorState((vector) => [updater(vector[0])]);
        }
        else {
            setVectorState([stateOrUpdater]);
        }
    };
    return [source, setState, sealState];
}
exports.createState = createState;
