"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createVectorState = void 0;
const retil_support_1 = require("retil-support");
const observe_1 = require("./observe");
function createVectorState(
// Doesn't accept a setter function, as it's not a hook that stores state
// between renders and thus it wouldn't make sense to do so.
initialState, 
// If provided, the state will only be updated if this function indicates
// that it is not equal to the current state.
isEqual = retil_support_1.areArraysShallowEqual) {
    let hasState = arguments.length !== 0;
    let state = initialState;
    let next = null;
    let seal = null;
    const setState = (stateOrUpdater) => {
        const nextState = typeof stateOrUpdater === 'function'
            ? stateOrUpdater(state)
            : stateOrUpdater;
        const shouldUpdate = !hasState || !isEqual || !isEqual(nextState, state);
        if (shouldUpdate) {
            hasState = true;
            state = nextState;
            if (next) {
                next(state);
            }
        }
    };
    const source = (0, observe_1.observe)((onNext, _, onSeal) => {
        next = onNext;
        seal = onSeal;
        if (hasState) {
            onNext(state);
        }
        return () => {
            next = null;
            seal = null;
        };
    });
    const sealState = () => {
        if (seal) {
            seal();
        }
    };
    return [source, setState, sealState];
}
exports.createVectorState = createVectorState;
