/* eslint-disable react-hooks/rules-of-hooks */
/// <reference types="react/experimental" />
import React, { useCallback, useMemo } from 'react';
import { getSnapshotPromise, hasSnapshot, nullSource, } from '../source';
const { unstable_createMutableSource: createMutableSource, unstable_useMutableSource: useMutableSource, } = React;
const MissingToken = Symbol();
const subscribe = ([, subscribe], cb) => subscribe(cb);
let mutableSources;
//
// TODO
// double check that I'm not relying on a promise being thrown by getVector anywhere
//
export const useSourceModern = (maybeSource, options = {}) => {
    const hasDefaultValue = 'defaultValue' in options;
    const { defaultValue, startTransition } = options;
    const [core, select] = maybeSource || nullSource;
    if (!mutableSources) {
        mutableSources = new WeakMap([
            [nullSource[0], createMutableSource(nullSource[0], nullSource[0][0])],
        ]);
    }
    const getSnapshot = useCallback((core) => {
        const snapshot = hasSnapshot([core, select])
            ? select(core[0]()[0])
            : MissingToken;
        if (snapshot === MissingToken && !hasDefaultValue) {
            throw getSnapshotPromise([core, select]);
        }
        return snapshot;
    }, [hasDefaultValue, select]);
    const subscribeWithTransition = useMemo(() => startTransition
        ? ([, subscribe], callback) => subscribe(() => {
            startTransition(callback);
        })
        : subscribe, [startTransition]);
    let mutableSource = mutableSources.get(core);
    if (!mutableSource) {
        mutableSource = createMutableSource(core, core[0]);
        mutableSources.set(core, mutableSource);
    }
    const value = useMutableSource(mutableSource, getSnapshot, subscribeWithTransition);
    return value === MissingToken || maybeSource === null ? defaultValue : value;
};
